Attribute VB_Name = "LibGeography"
Option Explicit

Sub DegToDMS(ByVal L As Double, D As Integer, M As Integer, S As Double)
On Error GoTo DegToDMS_Err
'
' Converts a decimal degree to Degrees, Minutes, and Seconds
' Seconds may contain up to 3 decimal places.
' e.g. 15.5 -> 15,30,0
'
  D = Int(L)
  L = (L - D) * 60
  M = Int(L)
  S = Val(Format((L - M) * 60, "#.###"))
  
Exit Sub
DegToDMS_Err:
ErrH App.ProductName & " code library", "Sub DegToDMS"
Exit Sub
End Sub
Function DegToDMSStr(ByVal L As Double) As String
On Error GoTo DegToDMSStr_Err
'
' Converts a decimal value to Degrees, Minutes, and Seconds
' Processes Seconds up to 3 decimal places.
' e.g. 15.5 -> 15 30' 0"
'
Dim D As Integer, M As Integer, S As Double
  D = Int(L)
  L = (L - D) * 60
  M = Int(L)
  S = Val(Format((L - M) * 60, "#.###"))
  DegToDMSStr = D & " " & M & "' " & S & """"
Exit Function
DegToDMSStr_Err:
ErrH App.ProductName & " code library", "Function DegToDMSStr"
Exit Function
End Function
Function DMSStrToDeg(ByVal DMS As String) As Double
On Error GoTo DMSStrToDeg_Err
'
' Converts a string value in the format [d m' s"] to a decimal number.
' Not all elements need be present, and they may contain decimal digits.
' e.g. 15 30' 0" -> 15.5
'
Dim i As Integer, w As String, Temp As Double
  Temp = 0
  For i = 1 To 3
    w = CutWord(DMS, DMS)
    Select Case Right(w, 1)
      Case "'"
        Temp = Temp + Val(w) / 60
      Case """"
        Temp = Temp + Val(w) / 3600
      Case Else
        Temp = Temp + Val(w)
    End Select
  Next i
  DMSStrToDeg = Temp
Exit Function
DMSStrToDeg_Err:
ErrH App.ProductName & " code library", "Function DMSStrToDeg"
Exit Function
End Function
Function DMSToDeg(D As Double, M As Double, S As Double) As Double
On Error GoTo DMSToDeg_Err
'
' Converts separate Degree, Minute, Second values into decimal degrees.
' e.g. 15,30,0 -> 15.5
'
  DMSToDeg = D + M / 60 + S / 3600
Exit Function
DMSToDeg_Err:
ErrH App.ProductName & " code library", "Function DMSToDeg"
Exit Function
End Function
Sub LatLongToXYZ(Lat As Double, Lon As Double, Radius As Double, X As Double, Y As Double, Z As Double)
On Error GoTo LatLongToXYZ_Err
'
' Converts Latitude, Longitude, Radius to 3d-Cartesian coordinates
'
' Assumes:
'   X axis runs through 270 (-X) and 90 (+X) Latitude
'   Y axis runs North (+Y) to South (-Y)
'   Z axis runs through 0 (-Z) and 180 (+Z) Latitude
'
  Y = Radius * Sin(Deg2Rad(Lat))
  X = Radius * Sin(Deg2Rad(Lon)) * Cos(Deg2Rad(Lat))
  Z = -Radius * Cos(Deg2Rad(Lon)) * Cos(Deg2Rad(Lat))
Exit Sub
LatLongToXYZ_Err:
ErrH App.ProductName & " code library", "Sub LatLongToXYZ"
Exit Sub
End Sub
Sub XYZToLatLong(X As Double, Y As Double, Z As Double, Lat As Double, Lon As Double, Radius As Double)
On Error GoTo XYZToLatLong_Err
'
' Converts 3d-Cartesian coordinates to Latitude, Longitude, and Radius
'
' Assumes:
'   X axis runs through 270 (-X) and 90 (+X) Latitude
'   Y axis runs North (+Y) to South (-Y)
'   Z axis runs through 0 (-Z) and 180 (+Z) Latitude
'
' Uses functions from the Trigonometry module
'
  Radius = Sqr(X * X + Y * Y + Z * Z)
  If Abs(Radius) < 0.0000000001 Then Radius = 0  ' Accomodate round-off error
  If Radius = 0 Then                             ' Zero radius has no other coordinates
    Radius = 0
    Lat = 0
    Lon = 0
  Else
    Lat = Rad2Deg(ArcSin(Y / Radius))
    If (Lat Mod 90) = 0 Then                     ' North/South pole has no longitude
      Lon = 0
    Else
      Lon = Rad2Deg(ATan2(-Z / Cos(Deg2Rad(Lat)), X / Cos(Deg2Rad(Lat))))
    End If
  End If
Exit Sub
XYZToLatLong_Err:
ErrH App.ProductName & " code library", "Sub XYZToLatLong"
Exit Sub
End Sub

