Attribute VB_Name = "LibMathBoolean"
Option Explicit

Sub ArrayBitReset(A() As Long, N As Integer)
On Error GoTo ArrayBitReset_Err
'
' Resets bit N in array A() - assumes 32 bits per element
' Return A() unchanged if invalid N
'
Dim Element As Integer
  Element = N \ 32 + LBound(A)
  If Element <= UBound(A) And N >= 0 Then BitReset A(Element), N Mod 32
Exit Sub
ArrayBitReset_Err:
ErrH App.ProductName & " code library", "Sub ArrayBitReset"
Exit Sub
End Sub

Sub ArrayBitSet(A() As Long, N As Integer)
On Error GoTo ArrayBitSet_Err
'
' Sets bit N in array A() - assumes 32 bits per element
' Return A() unchanged if invalid N
'
Dim Element As Integer
  Element = N \ 32 + LBound(A)
  If Element <= UBound(A) And N >= 0 Then BitSet A(Element), N Mod 32
Exit Sub
ArrayBitSet_Err:
ErrH App.ProductName & " code library", "Sub ArrayBitSet"
Exit Sub
End Sub

Function ArrayBitTest(A() As Long, N As Integer) As Integer
On Error GoTo ArrayBitTest_Err
'
' Sets bit N in array A() - assumes 32 bits per element
' Return A() unchanged if invalid N
'
Dim Element As Integer
  Element = N \ 32 + LBound(A)
  If Element <= UBound(A) And N >= 0 Then
    ArrayBitTest = BitTest(A(Element), N Mod 32)
  Else
    ArrayBitTest = False
  End If
Exit Function
ArrayBitTest_Err:
ErrH App.ProductName & " code library", "Function ArrayBitTest"
Exit Function
End Function

Sub ArrayBitFlip(A() As Long, N As Integer)
On Error GoTo ArrayBitFlip_Err
'
' Flips bit N in array A() - assumes 32 bits per element
' Return A() unchanged if invalid N
'
Dim Element As Integer
  Element = N \ 32 + LBound(A)
  If Element <= UBound(A) And N >= 0 Then BitFlip A(Element), N Mod 32
Exit Sub
ArrayBitFlip_Err:
ErrH App.ProductName & " code library", "Sub ArrayBitFlip"
Exit Sub
End Sub

Public Function SetBit(Num As Long, ByVal bit As Long) As Long
On Error GoTo SetBit_Err

If bit = 31 Then
      Num = &H80000000 Or Num
   Else
      Num = (2 ^ bit) Or Num
   End If
   
   SetBit = Num

Exit Function
SetBit_Err:
ErrH App.ProductName & " code library", "Public Function SetBit"
Exit Function

End Function
Public Function ClearBit(Num As Long, ByVal bit As Long) As Long
On Error GoTo ClearBit_Err

If bit = 31 Then
      Num = &H7FFFFFFF And Num
   Else
      Num = ((2 ^ bit) Xor &HFFFFFFFF) And Num
   End If
   
   ClearBit = Num

Exit Function
ClearBit_Err:
ErrH App.ProductName & " code library", "Public Function ClearBit"
Exit Function

End Function
Public Function IsBitSet(ByVal Num As Long, ByVal bit As Long) As Boolean
On Error GoTo IsBitSet_Err

IsBitSet = False
   
   If bit = 31 Then
      If Num And &H80000000 Then
         IsBitSet = True
      End If
   Else
      If Num And (2 ^ bit) Then
         IsBitSet = True
      End If
   End If

Exit Function
IsBitSet_Err:
ErrH App.ProductName & " code library", "Public Function IsBitSet"
Exit Function

End Function
Public Function IsEven(ByVal i As Long) As Boolean
On Error GoTo IsEven_Err

IsEven = Not -(i And 1)

Exit Function
IsEven_Err:
ErrH App.ProductName & " code library", "Public Function IsEven"
Exit Function

End Function
Public Function IsLeapYear(ByVal yr As Variant) As Boolean
On Error GoTo IsLeapYear_Err

If VarType(yr) = vbDate Then
      IsLeapYear = (Day(DateSerial(Year(yr), 2, 29)) = 29)
   Else
      IsLeapYear = (Day(DateSerial(yr, 2, 29)) = 29)
   End If

Exit Function
IsLeapYear_Err:
ErrH App.ProductName & " code library", "Public Function IsLeapYear"
Exit Function
End Function

Sub BitFlip(X As Long, N As Integer)
On Error GoTo BitFlip_Err
'
' Flips bit N in X
' Return X unchanged if invalid N
'
Dim Mask As Long
  Mask = BitMask(N)
'
' 2 methods
'
' X = IIf(X And Mask, X And Not Mask, X Or Mask)
  X = X And Not Mask Or (X And Not Mask Or Mask And Not X) And Mask
Exit Sub
BitFlip_Err:
ErrH App.ProductName & " code library", "Sub BitFlip"
Exit Sub
End Sub
Function BitMask(N As Integer) As Long
On Error GoTo BitMask_Err
'
' Returns a Mask that can be uset to set a bit.
' Returns 0 if N outside range 0..31.  This causes
' dependant functions to return DATA unchanged.
'
Dim i As Integer, Mask As Long
  If N < 0 Or N > 31 Then
    BitMask = 0
    Exit Function
  End If
  If N = 31 Then
    BitMask = -1
  Else
    Mask = 1
    For i = 1 To N
      Mask = Mask + Mask
    Next i
    BitMask = Mask
  End If
Exit Function
BitMask_Err:
ErrH App.ProductName & " code library", "Function BitMask"
Exit Function
End Function
Sub BitReset(X As Long, N As Integer)
On Error GoTo BitReset_Err
'
' Resets bit N in X
' Return X unchanged if invalid N
'
  X = X And Not BitMask(N)
Exit Sub
BitReset_Err:
ErrH App.ProductName & " code library", "Sub BitReset"
Exit Sub
End Sub
Sub BitSet(X As Long, N As Integer)
On Error GoTo BitSet_Err
'
' Sets bit N in X
' Return X unchanged if invalid N
'
  X = X Or BitMask(N)
Exit Sub
BitSet_Err:
ErrH App.ProductName & " code library", "Sub BitSet"
Exit Sub
End Sub
Function BitTest(X As Long, N As Integer) As Integer
On Error GoTo BitTest_Err
'
' Tests bit N in X - returns True or False
' Return False if invalid N
'
  BitTest = (X And BitMask(N)) <> 0
Exit Function
BitTest_Err:
ErrH App.ProductName & " code library", "Function BitTest"
Exit Function
End Function

Public Function IsOdd(ByVal i As Long) As Boolean
On Error GoTo IsOdd_Err

IsOdd = -(i And 1)

Exit Function
IsOdd_Err:
ErrH App.ProductName & " code library", "Public Function IsOdd"
Exit Function

End Function
