Attribute VB_Name = "LibSorting"
Option Explicit

Sub BubbleSort(A() As Integer)
On Error GoTo BubbleSort_Err

' Slowest Sort: n^2 comparisons
'
' Calling convention:
'   Redim A(1 To 20) as Integer
'   BubbleSort A()
'
Dim i As Integer, J As Integer, Low As Integer, Hi As Integer, Temp As Integer
  Low = LBound(A)
  Hi = UBound(A)
  For i = Low To Hi - 1
    For J = i + 1 To Hi
      If A(i) > A(J) Then
        Temp = A(i)
        A(i) = A(J)
        A(J) = Temp
      End If
    Next J
  Next i

Exit Sub
BubbleSort_Err:
ErrH App.ProductName & " code library", "Sub BubbleSort"
Exit Sub

End Sub
Sub BubbleSort2(A() As Integer)
On Error GoTo BubbleSort2_Err

' Slowest Sort: n^2 comparisons
' Implements early cut-off
'
' Calling convention:
'   Redim A(1 To 20) As Integer
'   BubbleSort2 A()
'
Dim i As Integer, J As Integer, Low As Integer, Hi As Integer
Dim Temp As Integer, Swapped As Integer
  Low = LBound(A)
  Hi = UBound(A)
  For i = Low To Hi - 1
    Swapped = False
    For J = i + 1 To Hi
      If A(i) > A(J) Then
        Temp = A(i)
        A(i) = A(J)
        A(J) = Temp
        Swapped = True
      End If
    Next J
    If Not Swapped Then Exit Sub
  Next i

Exit Sub
BubbleSort2_Err:
ErrH App.ProductName & " code library", "Sub BubbleSort2"
Exit Sub

End Sub
Sub QuickSort(A() As Integer, ByVal Low As Integer, ByVal Hi As Integer)
On Error GoTo QuickSort_Err

' Very fast sort: n Log n comparisons
'
' Calling convention:
'   Redim A(1 To 20) as Integer
'   QuickSort A(), 1, 20
'
Dim MidValue As Integer, i As Integer, J As Integer, Temp As Integer
  If Hi <= Low Then Exit Sub
  MidValue = A((Low + Hi) \ 2)
  i = Low
  J = Hi
  Do While i <= J
    If A(i) >= MidValue And A(J) <= MidValue Then
      Temp = A(i)
      A(i) = A(J)
      A(J) = Temp
      i = i + 1
      J = J - 1
    Else
      If A(i) < MidValue Then i = i + 1
      If A(J) > MidValue Then J = J - 1
    End If
  Loop
  QuickSort A(), Low, J
  QuickSort A(), i, Hi

Exit Sub
QuickSort_Err:
ErrH App.ProductName & " code library", "Sub QuickSort"
Exit Sub

End Sub
Sub ShellSort(A() As Integer)
On Error GoTo ShellSort_Err

' Very fast sort: 2n Log n comparisons
'
' Calling convention:
'   Redim A(1 To 20) as Integer
'   ShellSort A()
'
Dim i As Integer, J As Integer, Low As Integer, Hi As Integer
Dim Temp As Integer, Swapped As Integer
  Low = LBound(A)
  Hi = UBound(A)
  J = (Hi - Low + 1) \ 2
  Do While J > 0
    For i = Low To Hi - J
      If A(i) > A(i + J) Then
        Temp = A(i)
        A(i) = A(i + J)
        A(i + J) = Temp
      End If
    Next i
    For i = Hi - J To Low Step -1
      If A(i) > A(i + J) Then
        Temp = A(i)
        A(i) = A(i + J)
        A(i + J) = Temp
      End If
    Next i
    J = J \ 2
  Loop

Exit Sub
ShellSort_Err:
ErrH App.ProductName & " code library", "Sub ShellSort"
Exit Sub

End Sub
Sub TagBubbleSort(S() As String, A() As Integer)
On Error GoTo TagBubbleSort_Err

' Slowest Sort: n^2 comparisons
'
' Calling convention:
'   Redim S(1 to 20) as String, A(1 To 20) as Integer
'   TagBubbleSort S(), A()
'
' Sorting strings is very inefficient on memory and fragments it
' greatly.  By using a tag sort, you are swapping pointers or tags
' into the string array.  The calling function also needs to understand
' that it needs to use tags.
'
' Tag sorts can be adopted to other sort types by using the tag in comparisons
'
Dim i As Integer, J As Integer, Low As Integer, Hi As Integer, Temp As Integer
  Low = LBound(A)
  Hi = UBound(A)
'
' Initialize Tag array
'
  For i = Low To Hi
    A(i) = i
  Next i
'
' Sort
'
  For i = Low To Hi - 1
    For J = i + 1 To Hi
      If S(A(i)) > S(A(J)) Then   ' Test String Array via tag
        Temp = A(i)               ' Swap tag values
        A(i) = A(J)
        A(J) = Temp
      End If
    Next J
  Next i

Exit Sub
TagBubbleSort_Err:
ErrH App.ProductName & " code library", "Sub TagBubbleSort"
Exit Sub

End Sub
